/* UPDATEOP.C */
/* Copyright (C) ---- SYWARE inc ---- 1992 */

#include <windows.h>
#ifndef __SQL
typedef void FAR * HDBC;
#endif

/***************************************************************************/
/*
	How to Write a Custom Insert Operator DLL
	--- -- ----- - ------ ------ -------- ---

	This file is a template for the source code of a DLL that implements 
	a custom update operation for DataSync.

	Within this template:

		ORIGINAL refers to values at the time the subset was created.
		NEW refers to the values in the subset at update time.
		CURRENT refers to values in the source database at udpate 
			time.

	The only call you really have to worry about is updateop().  This
	will be called once for each value that changed in the subset
	whose update operator specifies this DLL.  This call is made
	during the "find changes" phase of update.

	When called, updateop() should form an ASCII string that contains
	a query which will update the source database appropriately
	when it is run.  It should write this string to a text file
	(the HFILE for this file is passed as a parameter).

	Within the template, an the implementation of the REPLACE update
	operator appears in the comments as an example (within the example, 
	the NULL value cases are ignored).

	The parameters to this call should contain everything you need to
	know about the value that changed (its ORIGINAL, NEW, and CURRENT 
	value; its datatype; etc.)

	If, for some reason, you wish to go back to the database holding
	either the ORIGINAL or NEW value to get additional information,
	an ODBC connection handle (HDBC) is provided.  
*/
/***************************************************************************/

/* The VALUE struct describes the value being updated */

#define NEWLINE "\015\012"

#define MAX_UPDATE_OPTION_NAME_SIZE 31
#define MAX_COLUMN_NAME_SIZE 31

#define DATATYPE_INVALID   0x000
#define DATATYPE_CHARACTER 0x001
	/* If newDatatype/originalDatatype is DATATYPE_CHARACTER then...
		 ...the NEW value is lpNewValue (a zero terminated string)
		 ...the ORIGINAL value is lpOriginalValue (a zero terminated string) */
#define DATATYPE_INTEGER   0x002
	/* If newDatatype/originalDatatype is DATATYPE_INTEGER then...
		 ...the NEW value is *((long FAR *) lpNewValue)
		 ...the ORIGINAL value is *((long FAR *) lpOriginalValue) */
#define DATATYPE_FLOAT     0x004
	/* If newDatatype/originalDatatype is DATATYPE_FLOAT then...
		 ...the NEW value is *((double FAR *) lpNewValue)
		 ...the ORIGINAL value is *((double FAR *) lpOriginalValue) */
#define DATATYPE_BCD       0x008
	/* If newDatatype/originalDatatype is DATATYPE_BCD then...
		 ...the NEW value is (newPrecision/2) bytes long (each
			byte containing two BCD digits and is pointed to by
			lpNewValue)
		 ...the ORIGINAL value is (originalPrecision/2) bytes long (each 
			byte containing two BCD digits and is pointed to by
			lpOriginalValue) */
#define DATATYPE_DATE      0x010
	/* If newDatatype/originalDatatype is DATATYPE_DATE then...
		 ...the NEW value is a zero terminated string pointed to by
			lpNewValue with the format {d yyyy-mm-dd}
			(including the starting and ending brackets).
		 ...the ORIGINAL value is a zero terminated string pointed to by
			lpOriginalValue with the format {d yyyy-mm-dd}
			(including the starting and ending brackets). */
#define DATATYPE_TIME      0x020
	/* If newDatatype/originalDatatype is DATATYPE_TIME then...
		 ...the NEW value is a zero terminated string pointed to by
			lpNewValue with the format {t hh:mm:ss.ssssss}
			(including the starting and ending brackets).
		 ...the ORIGINAL value is a zero terminated string pointed to by
			lpOriginalValue with the format {t hh:mm:ss.ssssss}
			(including the starting and ending brackets). */
#define DATATYPE_DATETIME  0x040
	/* If newDatatype/originalDatatype is DATATYPE_DATETIME then...
		 ...the NEW value is a zero terminated string pointed to by
			lpNewValue with the format {dt yyyy-mm-dd hh:mm:ss.ssssss}
			(including the starting and ending brackets).
		 ...the ORIGINAL value is a zero terminated string pointed to by
			lpOriginalValue with the format {dt yyyy-mm-dd hh:mm:ss.ssssss}
			(including the starting and ending brackets). */
#define DATATYPE_LOGICAL   0x080
	/* If newDatatype/originalDatatype is DATATYPE_LOGICAL then...
		 ...the NEW value is *((int FAR *) lpNewValue) where
			non-zero values represent TRUE and zero values represent 
			FALSE.
		 ...the ORIGINAL value is *((int FAR *) lpOriginalValue) where
			non-zero values represent TRUE and zero values represent 
			FALSE. */

typedef struct tagVALUE {
	BOOL        key;              /* Is this value a key column? */
	char        updateOption[MAX_UPDATE_OPTION_NAME_SIZE+1];
								  /* The name of the DLL (without the DLL ext) */
	char        newName[MAX_COLUMN_NAME_SIZE+1];
				  /* The column holding the NEW value */
	short       newDatatype;      /* The DATATYPE_* of the NEW value */
	long        newWidth;         /* The maximum width of the NEW value */
	long        newPrecision;     /* The precision of the NEW value (if BCD) */
	short       newScale;         /* The scale of the NEW value (if BCD) */
	char        originalName[MAX_COLUMN_NAME_SIZE+1];
								  /* The column holding the ORIGINAL value */
	short       originalDatatype; /* The DATATYPE_* of the ORIGINAL value */
	long        originalWidth;    /* The maximum width of the ORIGINAL value */
	long        originalPrecision;/* The precision of the ORIGINAL value (if BCD) */
	short       originalScale;    /* The scale of the ORIGINAL value (if BCD) */
	char        currentName[MAX_COLUMN_NAME_SIZE+1];
								  /* The column holding the CURRENT value */
	short       currentDatatype;  /* The DATATYPE_* of the CURRENT value */
	long        currentWidth;     /* The maximum width of the CURRENT value */
	long        currentPrecision; /* The precision of the CURENT value (if BCD) */
	short       currentScale;     /* The scale of the CURRENT value (if BCD) */
} VALUE;
typedef VALUE FAR* LPVALUE;

/****************************************************************************/

BOOL FAR PASCAL updateop (  /* This routine must be called "updateop()" */
/*
	PURPOSE:

	PARAMETERS:
*/
	HWND        hwnd,               /* Window handle */
	BOOL        fFlag,              /* If FALSE, the SQL should be */
									/*    in terms of the source */
									/*    database's table names, */
									/*    column names, etc. */
									/* If TRUE, the SQL should be */
									/*    written in terms of the */
									/*    subset's table names, column */
									/*    names, etc. */
	LPVALUE     lpValue,            /* Description of value being updated */

	HDBC        hdbcNew,            /* Connection to NEW value */
	LPSTR       lpNewTableName,     /* Name of file holding NEW value */
	LPSTR       lpNewKey,           /* SQL expression to locate row */
									/*    containing NEW value (i.e., */
									/*    " WHERE (NAME = 'Fred')" */
	BOOL        bNewIsNull,         /* Is NEW value null? */
	LPSTR       lpNewValue,         /* The NEW value */

	HDBC        hdbcOriginal,       /* Connection to ORIGINAL value */
	LPSTR       lpOriginalTableName,/* Name of file holding ORIGINAL */
									/*    value */
	LPSTR       lpOriginalKey,      /* SQL expression to locate row */
									/*    containing ORIGINAL value (i.e., */
									/*    " WHERE (NAME = 'Fred')" */
	BOOL        bOriginalIsNull,    /* Is ORIGINAL value null? */
	LPSTR       lpOriginalValue,    /* The ORIGINAL value */

	LPSTR       lpCurrentTableName, /* Name of table holding CURRENT */
									/*    value */
	LPSTR       lpCurrentKey,       /* SQL expression to locate row */
									/*    containing CURRENT value (i.e, */
									/*    " WHERE (NAME = 'Fred')" */

	HFILE       hfLogfile )         /* Logfile to write */
/*
	COMMENTS:

	none
*/
{
	/* <put code here that writes an SQL character string to hfLogfile.> */
	/* <The terminating CR/LF is required.> */

	/* A simple implementation of the REPLACE update operator (not taking
	   NULL values, long strings, or strings with embedded quotes into
	   account) would be:

	char buffer[20];

	_lwrite(hfLogfile, "UPDATE ", 7);
	if (fFlag)
		_lwrite(hfLogfile, lpNewTableName, lstrlen(lpNewTableName));
	else
		_lwrite(hfLogfile, lpCurrentTableName, lstrlen(lpCurrentTableName));
	_lwrite(hfLogfile, " SET ", 5);
	if (fFlag)
		_lwrite(hfLogfile, lpValue->newName, lstrlen(lpValue->newName));
	else
		_lwrite(hfLogfile, lpValue->currentName, lstrlen(lpValue->currentName));
	_lwrite(hfLogfile, " = ", 3);
	switch (lpValue->newDatatype) {

	case DATATYPE_CHARACTER:
		_lwrite(hfLogfile, "'", 1);
		_lwrite(hfLogfile, lpNewValue, lstrlen(lpNewValue));
		_lwrite(hfLogfile, "'", 1);
		break;

	case DATATYPE_INTEGER:
		wsprintf(buffer, "%ld", (*((long FAR *) lpNewValue)));
		_lwrite(hfLogfile, buffer, lstrlen(buffer));
		break;

	case DATATYPE_FLOAT:
		wsprintf((LPSTR) buffer, "%f", (*((double FAR *) lpNewValue)));
		_lwrite(hfLogfile, buffer, lstrlen(buffer));
		break;

	case DATATYPE_BCD:
		wsprintf((LPSTR) buffer, "%f", (*((double FAR *) lpNewValue)));
		_lwrite(hfLogfile, buffer, lstrlen(buffer));
		break;

	case DATATYPE_DATE:
	case DATATYPE_TIME:
	case DATATYPE_DATETIME:
		_lwrite(hfLogfile, lpNewValue, lstrlen(lpNewValue));
		break;
	
	case DATATYPE_LOGICAL:
		if (*((int FAR *) lpNewValue))
			_lwrite(hfLogfile, "1", 3);
		else
			_lwrite(hfLogfile, "0", 3);
		break;

	default:
		return FALSE;
	}
	
	if (fFlag)
		_lwrite(hfLogfile, lpNewKey, lstrlen(lpNewKey));
	else
		_lwrite(hfLogfile, lpCurrentKey, lstrlen(lpCurrentKey));

	_lwrite(hfLogfile, NEWLINE, lstrlen(NEWLINE));

	return TRUE;
*/
	/* Return TRUE for success.  FALSE for failure. */
	return TRUE;
}

/****************************************************************************/
#ifdef WIN32
BOOL WINAPI DllMain (
	HANDLE    hModule,
	DWORD     dwFunction,
	LPVOID    lpNot)
{
	switch (dwFunction) {
	case DLL_PROCESS_ATTACH:
		break;

	case DLL_PROCESS_DETACH:
		break;

	default:
		break;
	}
	return TRUE;
}
#else
int FAR PASCAL LibMain (
	HINSTANCE       hinst,
	WORD            wDataSeg,
	WORD            cbHeapSize,
	LPSTR           lpszCmdLine )
{
	if (cbHeapSize != 0)
		UnlockData(0);
	return 1;
}
int FAR PASCAL WEP (
	int             nParameter )
{
	return 1;
}

#endif
/****************************************************************************/
